// Accept ADSR param data from widgets, process this, and send it to both the curve~
// object that performs enveloping and to a function object that displays the envelope.
// Used by the Auzzie envelopr module.
autowatch = 1;
inlets = 5;
setinletassist(0, "attack time [msec]");
setinletassist(1, "decay time [msec]");
setinletassist(2, "sustain level [0,1]");
setinletassist(3, "release time [msec]");
setinletassist(4, "curvature [-1,1]");
outlets = 2;
setoutletassist(0, "messages for function object");
setoutletassist(1, "messages for curve~, prepended with 1 for noteon, 0 for noteoff lists");

var adsr = new Array(1., 1., 1., 1.);	// stores values from Max (probably widgets)
var noteon = new Array(1, 1., 1., 0.5, 1., 0., -0.5);
var noteoff = new Array(0, 0., 1., -0.5);
var curvature = 0.5;							// curvature term for curve~

// Maximum values for A, D, and R stages, plus enough to see sustain stage
// These must match widget ranges in GUI.
var kSustainWidth = 1000;
var kGraphWidth = 5000 + 2000 + kSustainWidth + 8000;

function msg_int(i)
{
	msg_float(i);
}

function msg_float(f)
{
	if (inlet < 0 || inlet > 4) {
		error("az1.sub-adsr-display.js: inlet number out of range\n");
		return;
	}
	if (inlet == 4)
		curvature = f;
	else
		adsr[inlet] = f;
	update_envelope_params();
	update_function_display();
}

update_envelope_params.local = 1;
function update_envelope_params()
{
	noteon[2] = adsr[0];		// attack time
	noteon[3] = curvature;
	noteon[4] = adsr[2];		// sustain level
	noteon[5] = adsr[1];		// decay time
	noteon[6] = -curvature;
	noteoff[2] = adsr[3];	// release time
	noteoff[3] = -curvature;
	outlet(1, noteoff);
	outlet(1, noteon);
}

// Send messages to the function object to show ADSR params with one
// curvature for A, D, and R stages.
// Graph is set to a constant width, because it turns out to be much
// clearer to understand envelope stage durations this way, rather than
// always scaling the graph domain to make the graph fit the total width
// of the function display, regardless of total duration.
// Note that the sustain stage does not have a duration, since this
// is determined by the time difference between note-on and note-off
// events. But we give it a constant graph duration to make it visible.
update_function_display.local = 1;
function update_function_display()
{
	point1_x = adsr[0];
	point2_x = point1_x + adsr[1];
	point3_x = point2_x + kSustainWidth;
	point4_x = point3_x + adsr[3];

	// total width of graph (redundant to keep sending this constant)
	outlet(0, "domain", kGraphWidth);
	
	// msg: zero-based breakpoint index, x val, y val
	outlet(0, 0, 0, 0.);						// 1st breakpoint 
	outlet(0, 1, point1_x, 1.);			// end of attack stage
	outlet(0, 2, point2_x, adsr[2]);		// end of decay stage
	outlet(0, 3, point3_x, adsr[2]);		// end of sustain stage
	outlet(0, 4, point4_x, 0.);			// end of release stage

	outlet(0, "setcurve", 1, curvature);	// A
	outlet(0, "setcurve", 2, -curvature);	// D
	outlet(0, "setcurve", 4, -curvature);	// R
}

